import unittest
import time
from nose.tools import eq_
from django.utils.importlib import import_module
from session import SessionStore as MultiSession


class TestMultiSessions(unittest.TestCase):

    def setUp(self):
        self.multi_session = MultiSession()

    def test_modify_and_keys(self):
        eq_(self.multi_session.modified, False)
        self.multi_session.create()
        self.multi_session['test'] = 'test_me'
        eq_(self.multi_session.modified, True)
        eq_(self.multi_session['test'], 'test_me')

    def test_save_and_delete(self):
        self.multi_session['key'] = 'value'
        self.multi_session.save()
        self.multi_session.exists(self.multi_session.session_key)
        eq_(self.multi_session.exists(self.multi_session.session_key), True)
        self.multi_session.delete(self.multi_session.session_key)
        eq_(self.multi_session.exists(self.multi_session.session_key), False)

    def test_flush(self):
        self.multi_session['key'] = 'another_value'
        self.multi_session.save()
        key = self.multi_session.session_key
        self.multi_session.flush()
        eq_(self.multi_session.exists(key), False)

    def test_items(self):
        self.multi_session['item1'], self.multi_session['item2'] = 1, 2
        self.multi_session.save()
        eq_(self.multi_session.items(), [('item2', 2), ('item1', 1)])

    def test_expiry(self):
        self.multi_session.set_expiry(1)
        eq_(self.multi_session.get_expiry_age(), 1)
        self.multi_session['key'] = 'expiring_value'
        self.multi_session.save()
        key = self.multi_session.session_key
        eq_(self.multi_session.exists(key), True)
        time.sleep(2)
        eq_(self.multi_session.exists(key), False)

    def test_save_and_load(self):
        self.multi_session.set_expiry(60)
        self.multi_session.setdefault('item_test', 777)
        self.multi_session.save()
        session_data = self.multi_session.load()
        eq_(session_data.get('item_test'), 777)
            
    def test_multiple_backends_separately(self):
        # Check the backend if more than two
        if len(self.multi_session.pool_backends) < 2:
            return None
        self.multi_session.create()
        session_key = self.multi_session.session_key
        for backend in self.multi_session.pool_backends:
            engine = import_module(backend['backend'])
            session = engine.SessionStore(session_key)
            if "write" not in backend["modes"]:
                eq_(session.exists(session_key), False)
            if "write" in backend["modes"]:
                eq_(session.exists(session_key), True)
            if "delete" in backend["modes"] and session.exists(session_key) is True:
                session.delete(session_key)
                eq_(session.exists(session_key), False)


if __name__ == '__main__':
    import os
    os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
    unittest.main()

# To run the test suite
# pip install nose
# python tests.py -v



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'test',
        'USER': 'test',
        'PASSWORD': 'test',
        'HOST': 'localhost',
        'PORT': '3306',
    }
}

SESSION_ENGINE = "multi_sessions.session"
SESSION_MULTISESSIONS_POOL = (
    {
        "backend": "redis_sessions.session",
        "modes": ["read", "write"],
    },
    {
        "backend": "django.contrib.sessions.backends.db",
        "modes": ["read", "delete"],
    },
)



from setuptools import setup, find_packages
from multi_sessions import __version__
setup(
    name='django-multi-sessions',
    version=__version__,
    description="Multi-sessions backend for Django",
    long_description="",
    keywords='django, sessions',
    author='Mikhail Andreev',
    author_email='x11org@gmail.com',
    url='http://github.com/adw0rd/django-multi-sessions',
    license='BSD',
    packages=find_packages(),
    zip_safe=False,
    install_requires=['setuptools', ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
)



# coding: utf-8
from django.conf import settings
from django.contrib.sessions.backends.base import SessionBase, CreateError
from django.utils.encoding import force_unicode
from django.utils.importlib import import_module


class SessionStore(SessionBase):
    """Multiple backends for your sessions!
    Designed for moving sessions from one session engine to another, without stopping the current sessions.
    It consistently passes backends in the list (settings.SESSION_MULTISESSIONS_POOL) and depending on the modes of doing the necessary.
    """
    def __init__(self, session_key=None):
        super(SessionStore, self).__init__(session_key)
        self.pool_backends = settings.SESSION_MULTISESSIONS_POOL

    def load(self):
        for backend in self.pool_backends:
            engine = import_module(backend['backend'])
            session = engine.SessionStore(self.session_key)
            if session.exists(self.session_key):
                session_data = session.load()
                return session_data
            else:
                # Try another backend
                continue
        # If the session is not exists, then create it
        self.create()
        return {}

    def exists(self, session_key):
        for backend in self.pool_backends:
            try:
                engine = import_module(backend['backend'])
                return engine.SessionStore(session_key).exists(session_key)
            except:
                # Try another backend
                continue
        return False

    def create(self):
        while True:
            self.session_key = self._get_new_session_key()
            try:
                # Save immediately to ensure we have a unique entry in the
                # database.
                self.save(must_create=True)
            except CreateError:
                # Key wasn't unique. Try again.
                continue
            self.modified = True
            self._session_cache = {}
            return

    def save(self, *a, **kw):
        backends = self._get_backends(modes=("write",))
        for backend in backends:
            try:
                engine = import_module(backend['backend'])
                session = engine.SessionStore(self.session_key)
                if hasattr(self, '_session_cache'):
                    if not hasattr(session, '_session_cache'):
                        session._session_cache = {}
                    session._session_cache.update(self._session_cache)
                session.save(*a, **kw)
            except:
                # Try another backend
                continue

    def delete(self, session_key=None):
        if session_key is None:
            if self._session_key is None:
                return
            session_key = self._session_key
        backends = self._get_backends(modes=("write", "delete"))
        for backend in backends:
            try:
                engine = import_module(backend['backend'])
                session = engine.SessionStore(session_key)
                session.delete()
            except:
                # Try another backend
                continue

    def _get_backends(self, modes=tuple()):
        """Return available backends by modes
        @return list
        """
        if not modes:
            return list(self.pool_backends)
        else:
            backends = []
            for backend in self.pool_backends:
                if any(map(lambda mode: mode in backend['modes'], modes)):
                    backends.append(backend)
            return backends
